#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <fftw3.h>

#include "../inductorwah.h"

#define ROWS 10000
#define COLS 11

    //FFTW variables
    fftw_complex fftw_in[ROWS], fftw_out[ROWS];
    fftw_plan ftPlanForward;

int main(int argc, char** argv)
{
    FILE * pImpulse, *pFFT;
    pImpulse = fopen ("impulse_response.txt","w");
    pFFT = fopen ("frequency_response.txt","w");

    size_t nfftFrameSize = ROWS;
    ftPlanForward = fftw_plan_dft_1d(nfftFrameSize, fftw_in, fftw_out, FFTW_FORWARD, FFTW_MEASURE);

    int k;
    for(k=0; k<nfftFrameSize; k++)
    {
        fftw_in[k][0] = 0.0;
        fftw_in[k][1] = 0.0;
    }

    //Sampling frequency
    float fs = 44100.0;
    //construct filter
    iwah_coeffs* wah = NULL;
    wah = make_iwah(wah, fs);
    iwah_circuit_preset(GCB, wah, fs);

    iwah_coeffs_fixed nwah;
    make_iwah_fixed_point(&nwah, wah);

    //output filter pulse responses
    int i,j;
    float gstep = 0.1;
    float gp = 0.0;
    float x = 0.0;
    float y = 0.0;
    unsigned int seq = 0;
    float impulse[ROWS+1][COLS+2];
    float fft[ROWS+1][COLS];
    float ftfreq[ROWS];
    float real = 0.0;
    float imag = 0.0;
    float fstep = fs/ROWS;

    for(i=0; i<COLS; i++)
    {
        for(j=0; j<ROWS; j++)
        {
                impulse[j][i] = 0.0;
        }
    }
    fprintf(pImpulse, "SEQ\tX\t");
    for(i=0; i<COLS; i++)
    {
        fprintf(pImpulse, "Y%d\t", i);
        for(j=0; j<ROWS; j++)
        {
            int32_t nx = 0;
            int32_t ngp = (int32_t) (gp*((float)(1<<24)));
            if(j == 0 )
            {
                x = 1.0;
                nx = (1<<24);
            }
            else
            {
                x = 0.0;
            }
            //y = iwah_tick(x, gp, wah);
            iwah_tick_n_fixed_point(&nwah, &nx, &ngp, 1);
            y = ((float) nx)/((float)(1<<24));
            //printf("%f\n", y);
            if(i == 0) {
                impulse[j][0] = x;
                impulse[j][1] = y;

                ftfreq[j] = ((float) j)*fstep;
            }
            else {
                impulse[j][i+1] = y;
            }

                fftw_in[j][0] = y;
                fftw_in[j][1] = 0.0;
        }

        fftw_execute(ftPlanForward);
        for(j=0; j<ROWS; j++)
        {
            real = fftw_out[j][0];
            imag = fftw_out[j][1];
            fft[j][i] = 20.0*log10( sqrtf(real*real + imag*imag) );

        }
        printf("Processing Step: %f\n", gp);
        gp += gstep;
        x = 0.0;
    }
    fprintf(pImpulse, "\n");


    for(j=0; j<ROWS; j++)
    {
        fprintf(pImpulse, "%d\t", j);

        if(ftfreq[j] > 100.0 && ftfreq[j] < 10000.0)
            fprintf( pFFT , "%f\t", ftfreq[j]);

        for(i=0; i<COLS; i++)
        {
            fprintf(pImpulse, "%f\t", impulse[j][i]);

            if(ftfreq[j] > 100.0 && ftfreq[j] < 10000.0)
                fprintf( pFFT , "%f\t", fft[j][i]);
        }
        fprintf(pImpulse, "\n");
        if(ftfreq[j] > 100.0 && ftfreq[j] < 10000.0)
            fprintf( pFFT , "\n");
    }

    free(wah);
    fftw_destroy_plan(ftPlanForward);
    fclose(pImpulse);
    fclose(pFFT);

    return 0;
}
